#include "glwidget.h"

#include <QOpenGLShaderProgram>
#include <QPainter>

GLWidget::GLWidget(QWidget *parent) :
    QOpenGLWidget(parent),
    m_program(0){

}

GLWidget::~GLWidget() {

}

void GLWidget::initializeGL() {
    // Creiamo l'oggetto QOpenGLShaderProgram
    m_program = new QOpenGLShaderProgram(this);

    // Compiliamo lo shader dei vertici
    if (!m_program->addShaderFromSourceFile(QOpenGLShader::Vertex, ":/vertex.glsl"))
        close();

    // Compiliamo il fragment shader
    if (!m_program->addShaderFromSourceFile(QOpenGLShader::Fragment, ":/fragment.glsl"))
        close();

    // Linkiamo gli shaders
    if (!m_program->link())
        close();

    // Localizzaziamo gli attributi negli shader
    m_posAttr = m_program->attributeLocation("posAttr");
    m_colAttr = m_program->attributeLocation("colAttr");
    m_matrixUniform = m_program->uniformLocation("matrix");
}

void GLWidget::resizeGL(int w, int h) {
    glViewport(0, 0, w, h);
}

void GLWidget::paintGL() {
    QPainter painter;
    painter.begin(this);

    painter.beginNativePainting();

    makeCurrent();

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);

    m_program->bind();

    QMatrix4x4 matrix;
    matrix.perspective(60, 4.0/3.0, 0.1, 100.0);
    matrix.translate(0, 0, -2);

    m_program->setUniformValue(m_matrixUniform, matrix);

    QVector2D vertices [] = {
        QVector2D(0.0f, 0.707f),
        QVector2D(-0.5f, -0.5f),
        QVector2D(0.5f, -0.5f)
    };

    m_program->enableAttributeArray( m_posAttr );
    m_program->setAttributeArray( m_posAttr, vertices );

    QVector3D colors[] = {
        QVector3D(1.0f, 0.0f, 0.0f),
        QVector3D(0.0f, 1.0f, 0.0f),
        QVector3D(0.0f, 0.0f, 1.0f)
    };

    m_program->enableAttributeArray( m_colAttr );
    m_program->setAttributeArray( m_colAttr, colors );

    glDrawArrays(GL_TRIANGLES, 0, 3);

    m_program->release();

    painter.endNativePainting();

    painter.setPen( Qt::white );
    painter.drawText( width()/2, height()/2, "QOpenGLWidget in azione!");

    painter.end();
}
